# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'AngleWidget.ui'
#
# Created by: PyQt5 UI code generator 5.14.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AngleWidget(object):
    def setupUi(self, AngleWidget):
        AngleWidget.setObjectName("AngleWidget")
        AngleWidget.resize(342, 70)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AngleWidget.sizePolicy().hasHeightForWidth())
        AngleWidget.setSizePolicy(sizePolicy)
        self.gridLayout = QtWidgets.QGridLayout(AngleWidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.spin = QtWidgets.QDoubleSpinBox(AngleWidget)
        self.spin.setEnabled(False)
        self.spin.setSuffix("°")
        self.spin.setMinimum(-90.0)
        self.spin.setMaximum(90.0)
        self.spin.setObjectName("spin")
        self.gridLayout.addWidget(self.spin, 0, 2, 1, 1)
        self.label = QtWidgets.QLabel(AngleWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 1, 1, 1)
        self.radio = QtWidgets.QRadioButton(AngleWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.radio.sizePolicy().hasHeightForWidth())
        self.radio.setSizePolicy(sizePolicy)
        self.radio.setText("")
        self.radio.setObjectName("radio")
        self.gridLayout.addWidget(self.radio, 0, 0, 1, 1)
        self.slider = QtWidgets.QSlider(AngleWidget)
        self.slider.setEnabled(False)
        self.slider.setMinimum(-90)
        self.slider.setMaximum(90)
        self.slider.setOrientation(QtCore.Qt.Horizontal)
        self.slider.setTickPosition(QtWidgets.QSlider.TicksBelow)
        self.slider.setTickInterval(10)
        self.slider.setObjectName("slider")
        self.gridLayout.addWidget(self.slider, 1, 0, 1, 3)

        self.retranslateUi(AngleWidget)
        QtCore.QMetaObject.connectSlotsByName(AngleWidget)

    def retranslateUi(self, AngleWidget):
        _translate = QtCore.QCoreApplication.translate
        AngleWidget.setWindowTitle(_translate("AngleWidget", "Form"))
        self.label.setText(_translate("AngleWidget", "TextLabel"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'Ui_PathwaysWidget.ui'
#
# Created by: PyQt5 UI code generator 5.14.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PathwaysWidget(object):
    def setupUi(self, PathwaysWidget):
        PathwaysWidget.setObjectName("PathwaysWidget")
        PathwaysWidget.resize(326, 390)
        self.verticalLayout = QtWidgets.QVBoxLayout(PathwaysWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setContentsMargins(-1, 9, -1, -1)
        self.gridLayout.setObjectName("gridLayout")
        self.molecule_combo = QtWidgets.QComboBox(PathwaysWidget)
        self.molecule_combo.setCurrentText("")
        self.molecule_combo.setObjectName("molecule_combo")
        self.gridLayout.addWidget(self.molecule_combo, 0, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(PathwaysWidget)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 3, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(PathwaysWidget)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.j_spin = QtWidgets.QSpinBox(PathwaysWidget)
        self.j_spin.setProperty("value", 10)
        self.j_spin.setObjectName("j_spin")
        self.gridLayout.addWidget(self.j_spin, 2, 1, 1, 1)
        self.update_model = QtWidgets.QPushButton(PathwaysWidget)
        self.update_model.setObjectName("update_model")
        self.gridLayout.addWidget(self.update_model, 4, 0, 1, 2)
        self.label = QtWidgets.QLabel(PathwaysWidget)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.k_spin = QtWidgets.QSpinBox(PathwaysWidget)
        self.k_spin.setProperty("value", 1)
        self.k_spin.setObjectName("k_spin")
        self.gridLayout.addWidget(self.k_spin, 3, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(PathwaysWidget)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 1, 0, 1, 1)
        self.direction_combo = QtWidgets.QComboBox(PathwaysWidget)
        self.direction_combo.setObjectName("direction_combo")
        self.gridLayout.addWidget(self.direction_combo, 1, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.label_5 = QtWidgets.QLabel(PathwaysWidget)
        self.label_5.setObjectName("label_5")
        self.verticalLayout.addWidget(self.label_5)
        self.pw_list = ListView(PathwaysWidget)
        self.pw_list.setObjectName("pw_list")
        self.verticalLayout.addWidget(self.pw_list)

        self.retranslateUi(PathwaysWidget)
        self.molecule_combo.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(PathwaysWidget)

    def retranslateUi(self, PathwaysWidget):
        _translate = QtCore.QCoreApplication.translate
        PathwaysWidget.setWindowTitle(_translate("PathwaysWidget", "Form"))
        self.label_3.setText(_translate("PathwaysWidget", "K"))
        self.label_2.setText(_translate("PathwaysWidget", "J"))
        self.update_model.setText(_translate("PathwaysWidget", "Update list"))
        self.label.setText(_translate("PathwaysWidget", "Molecule"))
        self.label_4.setText(_translate("PathwaysWidget", "Direction"))
        self.label_5.setText(_translate("PathwaysWidget", "Rotationally coherent peaks/pathways:"))
from .ListView import ListView



from PyQt5 import QtWidgets
from .Ui_PathwaysWidget import Ui_PathwaysWidget

class PathwaysWidget(QtWidgets.QWidget, Ui_PathwaysWidget):
    def __init__(self, parent=None):
        super(PathwaysWidget, self).__init__(parent)
        self.setupUi(self)



"""Calculate list of 2D peaks of 2D spectrum"""
from pathlib import Path
import string
import sys
from argparse import ArgumentParser
from pprint import pprint
from typing import List, Sequence

import rotsim2d.dressedleaf as dl
import rotsim2d.pathways as pw
import rotsim2d.propagate as prop
import toml
from asteval import Interpreter


class HelpfulParser(ArgumentParser):
    def error(self, message):
        sys.stderr.write('error: {:s}\n'.format(message))
        self.print_help()
        sys.exit(2)


def named_fields(s: str) -> List[str]:
    return [x[1] for x in string.Formatter().parse(s)
            if x[1] is not None]


def run():
    parser = HelpfulParser(
        description="Calculate and save to file list of 2D peaks or 2D spectrum.",
        add_help=False)
    parser.add_argument("input_paths", nargs='+',
                        help="Paths to input files.",)
    args = parser.parse_args()
    aeval = Interpreter(use_numpy=False, minimal=True)
    for input_path in args.input_paths:
        params = toml.load(input_path)
        pprint(params)

        if str in [type(x) for x in params['spectrum']['angles']]:
            params['spectrum']['angles'] = \
                [aeval(angle) for angle in params['spectrum']['angles']]

        if params['spectrum']['type'] == 'peaks':
            print("Calculating peak list...")
            peaks = dl.run_peak_list(params)
            print("Saving to {!s}...".format(params['output']['file']))
            peaks.to_file(params['output']['file'],
                          metadata=params)
        elif params['spectrum']['type'] in ('lineshapes', 'time'):
            print("Preparing DressedPathway's...")
            dls = dl.DressedPathway.from_params_dict(params['pathways'])
            print("Calculating 2D spectrum...")
            params = prop.run_update_metadata(params)

            if isinstance(params['spectrum']['pressure'], Sequence) and\
               'p' not in named_fields(params['output']['file']):
                raise ValueError(
                    "Format specifier with field 'p' not provided. "
                    "Data for all pressures would have been overwritten.")

            if not isinstance(params['spectrum']['pressure'], Sequence):
                pressures = [params['spectrum']['pressure']]
                if 'file' not in params['output']:
                    params['output']['file'] = Path(input_path).with_suffix('.h5')
            else:
                pressures = params['spectrum']['pressure'][:]
                if 'file' not in params['output']:
                    params['output']['file'] = Path(input_path).stem +\
                        '_{:.1f}.h5'
            for p in pressures:
                params['spectrum']['pressure'] = p
                print("Pressure = {:.2f} atm".format(p))
                fs_pu, fs_pr, spec2d = prop.run_propagate(
                    dls, params['spectrum'])
                output_file = params['output']['file'].format(p=p)
                print("Saving to {!s}...".format(output_file))
                prop.run_save(
                    output_file,
                    fs_pu, fs_pr, spec2d,
                    params)


if __name__ == '__main__':
    run()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'Ui_WaitingTimeWindow.ui'
#
# Created by: PyQt5 UI code generator 5.14.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_WaitingTimeWindow(object):
    def setupUi(self, WaitingTimeWindow):
        WaitingTimeWindow.setObjectName("WaitingTimeWindow")
        WaitingTimeWindow.resize(800, 600)
        self.centralwidget = QtWidgets.QWidget(WaitingTimeWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setUsesScrollButtons(False)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.tab)
        self.horizontalLayout_2.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout_2.setSpacing(2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.pws_widget = PathwaysWidget(self.tab)
        self.pws_widget.setObjectName("pws_widget")
        self.horizontalLayout_2.addWidget(self.pws_widget)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.tab_2)
        self.horizontalLayout_3.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout_3.setSpacing(2)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.diagrams = QtWidgets.QTextEdit(self.tab_2)
        font = QtGui.QFont()
        font.setFamily("Monospace")
        font.setPointSize(10)
        self.diagrams.setFont(font)
        self.diagrams.setReadOnly(True)
        self.diagrams.setAcceptRichText(False)
        self.diagrams.setTextInteractionFlags(QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.diagrams.setObjectName("diagrams")
        self.horizontalLayout_3.addWidget(self.diagrams)
        self.tabWidget.addTab(self.tab_2, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.plot_print_button = QtWidgets.QPushButton(self.centralwidget)
        self.plot_print_button.setObjectName("plot_print_button")
        self.verticalLayout.addWidget(self.plot_print_button)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.plots_widget = PlotsWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.plots_widget.sizePolicy().hasHeightForWidth())
        self.plots_widget.setSizePolicy(sizePolicy)
        self.plots_widget.setObjectName("plots_widget")
        self.horizontalLayout.addWidget(self.plots_widget)
        WaitingTimeWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(WaitingTimeWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 23))
        self.menubar.setObjectName("menubar")
        WaitingTimeWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(WaitingTimeWindow)
        self.statusbar.setObjectName("statusbar")
        WaitingTimeWindow.setStatusBar(self.statusbar)

        self.retranslateUi(WaitingTimeWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(WaitingTimeWindow)

    def retranslateUi(self, WaitingTimeWindow):
        _translate = QtCore.QCoreApplication.translate
        WaitingTimeWindow.setWindowTitle(_translate("WaitingTimeWindow", "Waiting time"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("WaitingTimeWindow", "Pathways"))
        self.diagrams.setHtml(_translate("WaitingTimeWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Monospace\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'monospace\';\"><br /></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("WaitingTimeWindow", "Diagrams"))
        self.plot_print_button.setText(_translate("WaitingTimeWindow", "Plot coherences/print diagrams"))
from .PathwaysWidget import PathwaysWidget
from .PlotsWidget import PlotsWidget



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'Ui_PlotsWidget.ui'
#
# Created by: PyQt5 UI code generator 5.14.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PlotsWidget(object):
    def setupUi(self, PlotsWidget):
        PlotsWidget.setObjectName("PlotsWidget")
        PlotsWidget.resize(701, 472)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(PlotsWidget.sizePolicy().hasHeightForWidth())
        PlotsWidget.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(PlotsWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.mpl = MplWidget(PlotsWidget)
        self.mpl.setObjectName("mpl")
        self.verticalLayout.addWidget(self.mpl)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.xlims_label = QtWidgets.QLabel(PlotsWidget)
        self.xlims_label.setObjectName("xlims_label")
        self.horizontalLayout.addWidget(self.xlims_label)
        self.xmin_spin = QtWidgets.QDoubleSpinBox(PlotsWidget)
        self.xmin_spin.setSingleStep(0.05)
        self.xmin_spin.setObjectName("xmin_spin")
        self.horizontalLayout.addWidget(self.xmin_spin)
        self.xmax_spin = QtWidgets.QDoubleSpinBox(PlotsWidget)
        self.xmax_spin.setSingleStep(0.05)
        self.xmax_spin.setProperty("value", 2.0)
        self.xmax_spin.setObjectName("xmax_spin")
        self.horizontalLayout.addWidget(self.xmax_spin)
        self.real_radio = QtWidgets.QRadioButton(PlotsWidget)
        self.real_radio.setChecked(True)
        self.real_radio.setObjectName("real_radio")
        self.horizontalLayout.addWidget(self.real_radio)
        self.imag_radio = QtWidgets.QRadioButton(PlotsWidget)
        self.imag_radio.setObjectName("imag_radio")
        self.horizontalLayout.addWidget(self.imag_radio)
        self.holdy_check = QtWidgets.QCheckBox(PlotsWidget)
        self.holdy_check.setObjectName("holdy_check")
        self.horizontalLayout.addWidget(self.holdy_check)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.update_plot_button = QtWidgets.QPushButton(PlotsWidget)
        self.update_plot_button.setObjectName("update_plot_button")
        self.horizontalLayout.addWidget(self.update_plot_button)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(PlotsWidget)
        QtCore.QMetaObject.connectSlotsByName(PlotsWidget)

    def retranslateUi(self, PlotsWidget):
        _translate = QtCore.QCoreApplication.translate
        PlotsWidget.setWindowTitle(_translate("PlotsWidget", "Form"))
        self.xlims_label.setText(_translate("PlotsWidget", "X limits (1/B)"))
        self.real_radio.setText(_translate("PlotsWidget", "Real"))
        self.imag_radio.setText(_translate("PlotsWidget", "Imaginary"))
        self.holdy_check.setText(_translate("PlotsWidget", "Hold Y"))
        self.update_plot_button.setText(_translate("PlotsWidget", "Update plot"))
from .MplWidget import MplWidget



from PyQt5 import QtWidgets, QtCore
from .Ui_PlotsWidget import Ui_PlotsWidget

class PlotsWidget(QtWidgets.QWidget, Ui_PlotsWidget):
    def __init__(self, parent=None):
        super(PlotsWidget, self).__init__(parent)
        self.setupUi(self)

        self.fig = self.mpl.canvas.fig
        self.gs = self.fig.add_gridspec(
            ncols=2, nrows=1, width_ratios=[1, 1])
        self.axes = [self.fig.add_subplot(self.gs[i]) for i in (0, 1)]
        self.secaxes = [ax.secondary_xaxis('top', functions=(
            lambda x: x, lambda x: x)) for ax in self.axes]
        self.axes[0].set_title('Individual', loc='left')
        self.axes[1].set_title("Sum", loc='left')
        for ax in self.axes:
            ax.set(xlabel='Time (1/B)', ylabel='Amplitude')
            ax.margins(0.0)
            ax.autoscale(True, 'both', True)
        for ax in self.secaxes:
            ax.set_xlabel('Time (ps)')
        self.fig.set_constrained_layout_pads(
            wspace=0.02, hspace=0.02)

        self.holdy_check.setChecked(False)
        self.holdy_check.stateChanged.connect(self.handle_holdy)

    @QtCore.pyqtSlot(int)
    def handle_holdy(self, state):
        if state == QtCore.Qt.CheckState.Checked:
            for ax in self.axes:
                ax.autoscale(False, 'y', None)
                ax.autoscale(True, 'x', None)
        elif state == QtCore.Qt.CheckState.Unchecked:
            for ax in self.axes:
                ax.autoscale(True, 'both', None)



# -*- coding: utf-8 -*-
import matplotlib
import matplotlib.cm as cm
import matplotlib.colors as clrs
import numpy as np
from matplotlib.backends.backend_qt5agg import \
    NavigationToolbar2QT as NavigationToolbar
from matplotlib.colorbar import Colorbar
from matplotlib.widgets import MultiCursor
from PyQt5 import QtWidgets

from ..MplCanvas import MplCanvas

nature_fontsize = 8
nature_rc = {
    # 'font.sans-serif': 'Arial',
    'font.size': nature_fontsize,
    'axes.labelsize': nature_fontsize,
    'xtick.labelsize': nature_fontsize,
    'ytick.labelsize': nature_fontsize,
    'legend.fontsize': nature_fontsize,
    'lines.markersize': 1.0,
    'lines.linewidth': 0.6,
    'xtick.major.size': 3,
    'xtick.minor.size': 1.5,
    'xtick.major.pad': 4,
    'ytick.major.size': 3,
    'ytick.major.pad': 4
}
matplotlib.rcParams.update(nature_rc)


class BlitManager:
    def __init__(self, canvas, animated_artists=()):
        """
        Parameters
        ----------
        canvas : FigureCanvasAgg
            The canvas to work with, this only works for sub-classes of the Agg
            canvas which have the `~FigureCanvasAgg.copy_from_bbox` and
            `~FigureCanvasAgg.restore_region` methods.

        animated_artists : Iterable[Artist]
            List of the artists to manage
        """
        self.canvas = canvas
        self._bg = None
        self._artists = []

        for a in animated_artists:
            self.add_artist(a)
        # grab the background on every draw
        self.cid = canvas.mpl_connect("draw_event", self.on_draw)

    def on_draw(self, event):
        """Callback to register with 'draw_event'."""
        cv = self.canvas
        if event is not None:
            if event.canvas != cv:
                raise RuntimeError
        self._bg = cv.copy_from_bbox(cv.figure.bbox)
        self._draw_animated()

    def add_artist(self, art):
        """
        Add an artist to be managed.

        Parameters
        ----------
        art : Artist

            The artist to be added.  Will be set to 'animated' (just
            to be safe).  *art* must be in the figure associated with
            the canvas this class is managing.

        """
        if art.figure != self.canvas.figure:
            raise RuntimeError
        art.set_animated(True)
        self._artists.append(art)

    def _draw_animated(self):
        """Draw all of the animated artists."""
        fig = self.canvas.figure
        for a in self._artists:
            fig.draw_artist(a)

    def update(self):
        """Update the screen with animated artists."""
        cv = self.canvas
        fig = cv.figure
        # paranoia in case we missed the draw event,
        if self._bg is None:
            self.on_draw(None)
        else:
            # restore the background
            cv.restore_region(self._bg)
            # draw all of the animated artists
            self._draw_animated()
            # update the GUI state
            cv.blit(fig.bbox)
        # let the GUI event loop process anything it has to do
        cv.flush_events()

    def suspend(self):
        """Remove animated property from artists."""
        for art in self._artists:
            art.set_animated(False)

    def resume(self):
        """Add animated property to artists."""
        for art in self._artists:
            art.set_animated(True)


class PolarizationWidget(QtWidgets.QWidget):
    def __init__(self, parent=None):
        QtWidgets.QWidget.__init__(self, parent)
        self.canvas = MplCanvas()
        self.toolbar = NavigationToolbar(self.canvas, parent)
        self.vbl = QtWidgets.QVBoxLayout()
        self.vbl.addWidget(self.canvas)
        self.vbl.addWidget(self.toolbar)
        self.setLayout(self.vbl)
        self.fig = self.canvas.fig
        self.figure_setup()

    @staticmethod
    def main_subplots(gs):
        for ss in gs:
            if ss.is_last_col():
                continue
            else:
                yield ss

    def figure_setup(self):
        # self.fig.patch.set_facecolor('white')
        fake_data = np.zeros((100, 100))
        self.gs = self.fig.add_gridspec(
            nrows=2, ncols=5, width_ratios=[20, 20, 20, 20, 1])
        subplots = list(self.main_subplots(self.gs))
        del subplots[3]
        self.axes = [self.fig.add_subplot(subplots[i]) for i in range(7)]
        self.set_axes_labels((r'$\Phi_3$', r'$\Phi_4$'))
        axcbar = self.fig.add_subplot(self.gs[:, -1])
        scalar_map = cm.ScalarMappable(norm=clrs.Normalize(-1, 1),
                                       cmap=cm.get_cmap('RdBu'))
        cbar = Colorbar(mappable=scalar_map,
                        ax=axcbar, orientation='vertical', extend='neither')
        extent = [-90, 90, -90, 90]
        self.images = [self.axes[i].imshow(
            fake_data, aspect='auto', extent=extent, origin='lower', vmin=-1.0,
            vmax=1.0, cmap=cm.get_cmap('RdBu')) for i in range(7)]
        self.fig.set_constrained_layout_pads(
            wspace=0.02, hspace=0.02)

        self.blit_manager = BlitManager(self.canvas, self.images)
        self.multicursor = MultiCursor(
            self.canvas, self.axes, useblit=True, horizOn=True, vertOn=True,
            color='gray', lw=0.7)
        self.canvas.mpl_connect('figure_leave_event', self.restore_main_blit)
        self.canvas.mpl_connect('figure_enter_event', self.disable_main_blit)

    def set_titles(self, titles):
        for title, ax in zip(titles, self.axes):
            ax.set_title(title, fontsize=8)

    def set_axes_labels(self, labels):
        ylabel, xlabel = labels
        for ax in self.axes:
            ax.set(xlabel=xlabel, ylabel=ylabel)

    def disable_main_blit(self, event):
        self.blit_manager.suspend()
        self.canvas.draw()

    def restore_main_blit(self, event):
        self.blit_manager.resume()
        self.canvas.restore_region(self.multicursor.background)
        self.canvas.blit()

    def figure_update(self, data):
        """`data` is a list of 2D arrays."""
        for i in range(7):
            self.images[i].set_data(data[i])
        self.blit_manager.update()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'PolarizationsUI.ui'
#
# Created by: PyQt5 UI code generator 5.14.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(735, 661)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.centrallayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.centrallayout.setObjectName("centrallayout")
        self.anglesLayout = QtWidgets.QVBoxLayout()
        self.anglesLayout.setObjectName("anglesLayout")
        self.centrallayout.addLayout(self.anglesLayout)
        self.mplwdg = PolarizationWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mplwdg.sizePolicy().hasHeightForWidth())
        self.mplwdg.setSizePolicy(sizePolicy)
        self.mplwdg.setMinimumSize(QtCore.QSize(200, 0))
        self.mplwdg.setObjectName("mplwdg")
        self.centrallayout.addWidget(self.mplwdg)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 735, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Polarization explorer"))
from .PolarizationWidget import PolarizationWidget



from PyQt5 import QtWidgets
from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import\
    FigureCanvasQTAgg as FigureCanvas

class MplCanvas(FigureCanvas):
    def __init__(self):
        self.fig = Figure(frameon=False)

        FigureCanvas.__init__(self, self.fig)
        FigureCanvas.setSizePolicy(self, QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)



from PyQt5 import QtWidgets, QtCore

class ListView(QtWidgets.QListView):
    def __init__(self, parent=None):
        super(ListView, self).__init__(parent)
        # self.setSizePolicy(
        #     QtWidgets.QSizePolicy.Policy.Maximum,
        #     self.sizePolicy().verticalPolicy())

    def sizeHint(self):
        return QtCore.QSize(
            self.sizeHintForColumn(0)+25,
            super(ListView, self).sizeHint().height())



import io
import itertools as it
import sys
from functools import partialmethod

import matplotlib as mpl
import numpy as np
import rotsim2d.dressedleaf as dl
import rotsim2d.pathways as pw
import rotsim2d.propagate as prop
import rotsim2d.symbolic.functions as sym
import rotsim2d.visual.functions as vis
from rotsim2d.rcpeaks import TBs, nth, RCPeaks
import scipy.constants as C
from molspecutils.molecule import CH3ClAlchemyMode, COAlchemyMode
from PyQt5 import QtCore, QtWidgets

from .Ui_WaitingTimeWindow import Ui_WaitingTimeWindow

nature_fontsize = 10
nature_rc = {
    # 'font.sans-serif': 'Arial',
    'font.size': nature_fontsize,
    'axes.labelsize': nature_fontsize,
    'xtick.labelsize': nature_fontsize,
    'ytick.labelsize': nature_fontsize,
    'legend.fontsize': nature_fontsize,
    'lines.markersize': 1.0,
    'lines.linewidth': 1.0,
    'xtick.major.size': 3,
    'xtick.minor.size': 1.5,
    'xtick.major.pad': 4,
    'ytick.major.size': 3,
    'ytick.major.pad': 4
}
mpl.rcParams.update(nature_rc)


class DressedPathwaysModel(QtCore.QAbstractListModel):
    def __init__(self, rc_peaks: RCPeaks, parent=None):
        QtCore.QAbstractListModel.__init__(self, parent)
        self.rc_peaks = rc_peaks

    def rowCount(self, parent=QtCore.QModelIndex()):
        return len(self.rc_peaks)

    def data(self, index, role):
        if not index.isValid():
            return None

        if index.row() >= self.rowCount():
            return None

        if role == QtCore.Qt.ItemDataRole.DisplayRole:
            return self.rc_peaks.render(index.row())
        else:
            return None


class WaitingTimeWindow(QtWidgets.QMainWindow, Ui_WaitingTimeWindow):
    def __init__(self):
        super(WaitingTimeWindow, self).__init__()
        self.setupUi(self)

        self.plots_widget.update_plot_button.clicked.connect(
            self.handle_update_plot)
        # set up pathways widget
        self.setup_pathwayswidget()

        self.update_model()
        self.pws_widget.pw_list.setCurrentIndex(
            self.dpmodel.index(0, 0))
        self.pws_widget.pw_list.activated.emit(
            self.dpmodel.index(0, 0))

    def setup_pathwayswidget(self):
        self.pws_widget.j_spin.valueChanged.connect(
            self.pws_widget.k_spin.setMaximum)
        self.pws_widget.j_spin.valueChanged.emit(
            self.pws_widget.j_spin.value())
        self.pws_widget.pw_list.activated.connect(
            self.plot_rcs)
        self.pws_widget.pw_list.activated.connect(
            self.update_sec_axes)
        self.pws_widget.pw_list.activated.connect(
            self.print_diagrams)
        self.plot_print_button.clicked.connect(
            self.handle_plot_print)

        self.pws_widget.molecule_combo.addItems(["CO", "CH3Cl"])
        self.pws_widget.direction_combo.addItems(["SII", "SI"])
        self.pws_widget.update_model.clicked.connect(self.update_model)

    @QtCore.pyqtSlot()
    def handle_update_plot(self):
        index = self.pws_widget.pw_list.currentIndex()
        if index.isValid():
            self.plot_rcs(index)

    @QtCore.pyqtSlot()
    def handle_plot_print(self):
        index = self.pws_widget.pw_list.currentIndex()
        if index.isValid():
            self.plot_rcs(index)
            self.update_sec_axes(index)
            self.print_diagrams(index)

    @QtCore.pyqtSlot(bool)
    def update_model(self, checked=True):
        molecule = self.pws_widget.molecule_combo.currentText()
        direction = self.pws_widget.direction_combo.currentText()
        j = self.pws_widget.j_spin.value()
        k = self.pws_widget.k_spin.value()
        self.dpmodel = DressedPathwaysModel(RCPeaks(molecule, direction, j, k))
        self.pws_widget.pw_list.setModel(self.dpmodel)

    @QtCore.pyqtSlot(QtCore.QModelIndex)
    def print_diagrams(self, index):
        buf = io.StringIO()
        def buf_print(s: str='', end='\n'):
            buf.write(s+end)
        pws = nth(iter(self.dpmodel.rc_peaks.dps), index.row())
        dl.pprint_dllist(pws, True, print=buf_print)
        self.diagrams.setPlainText(buf.getvalue())

    @QtCore.pyqtSlot(QtCore.QModelIndex)
    def update_sec_axes(self, index):
        molecule = self.pws_widget.molecule_combo.currentText()
        TB = TBs[molecule]
        for ax in self.plots_widget.secaxes:
            ax.set_functions((
                lambda tb_frac: tb_frac*TB*1e12,
                lambda tw: tw/TB/1e12))

    @QtCore.pyqtSlot(QtCore.QModelIndex)
    def plot_rcs(self, index):
        pws = nth(iter(self.dpmodel.rc_peaks.dps), index.row())
        molecule = self.pws_widget.molecule_combo.currentText()
        j = self.pws_widget.j_spin.value()
        TB = TBs[molecule]
        xmin = self.plots_widget.xmin_spin.value()
        xmax = self.plots_widget.xmax_spin.value()
        tws = np.linspace(xmin*TB, xmax*TB, 5000)
        resp = np.zeros((len(pws), tws.size), dtype=np.complex128)
        for i in range(len(pws)):
            resp[i] = prop.dressed_leaf_response(
                pws[i], [None, tws, None],
                ['t', 't', 't'], p=1e-4)

        ax1 = self.plots_widget.axes[1]
        while ax1.lines:
            ax1.lines.remove(ax1.lines[0])
        ax1.set_prop_cycle(None)
        if self.plots_widget.real_radio.isChecked():
            ax1.plot(tws/TB, resp.real.sum(axis=0), label='real')
        else:
            ax1.plot(tws/TB, resp.imag.sum(axis=0), label='imaginary')
        ax1.legend(loc='lower right', bbox_to_anchor=[1.0, 1.1], ncol=2)

        ax0 = self.plots_widget.axes[0]
        while ax0.lines:
            ax0.lines.remove(ax0.lines[0])
        # ax0.relim()
        ax0.set_prop_cycle(None)
        for i in range(len(pws)):
            label = '$'+vis.latex(sym.rcs_expression(pws[i].coherences[1], j))+'$'
            if self.plots_widget.real_radio.isChecked():
                ax0.plot(tws/TB, resp[i].real, label=label)
            else:
                ax0.plot(tws/TB, resp[i].imag, label=label)
        ax0.legend(loc='lower right', bbox_to_anchor=[1.0, 1.1], ncol=2)

        ax1.relim()
        ax0.relim()
        if self.plots_widget.holdy_check.isChecked():
            ax1.autoscale(True, "x", True)
            ax0.autoscale(True, "x", True)
        else:
            ax1.autoscale(True, "both", True)
            ax0.autoscale(True, "both", True)

        self.plots_widget.mpl.canvas.draw()

def run():
    app = QtWidgets.QApplication(sys.argv)
    mpl.rcParams['figure.dpi'] = app.desktop().physicalDpiX()
    mw = WaitingTimeWindow()
    mw.show()
    sys.exit(app.exec_())



from PyQt5 import QtWidgets
from ..MplCanvas import MplCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT

class MplWidget(QtWidgets.QWidget):
    def __init__(self, parent=None):
        QtWidgets.QWidget.__init__(self, parent)
        self.canvas = MplCanvas()
        self.toolbar = NavigationToolbar2QT(self.canvas, parent)
        self.vbl = QtWidgets.QVBoxLayout()
        self.vbl.addWidget(self.canvas)
        self.vbl.addWidget(self.toolbar)
        self.setLayout(self.vbl)



# * Imports
import argparse
import math
import sys
from argparse import ArgumentParser

import matplotlib as mpl
import matplotlib.colors as colors
import matplotlib.pyplot as plt
import numpy as np
import rotsim2d.dressedleaf as dl
import rotsim2d.pathways as pw
import rotsim2d.visual as vis
from asteval import Interpreter
from matplotlib.cm import get_cmap
from matplotlib.colorbar import Colorbar
from molspecutils.molecule import CH3ClAlchemyMode, COAlchemyMode


class HelpfulParser(ArgumentParser):
    def error(self, message):
        sys.stderr.write('error: {:s}\n'.format(message))
        self.print_help()
        sys.exit(2)


def run():
# * Parse arguments
    parser = HelpfulParser(
        description='Plot 2D resonance map of 2D spectrum of CO or CH3Cl.'
        ' Clicking on a resonance will print on standard output all pathways'
        ' contributing to it.',
        add_help=False)
    parser.add_argument('molecule', choices=('CO', 'CH3Cl'),
                        help="Molecule.")
    parser.add_argument('-h', '--help', action='help', default=argparse.SUPPRESS,
                        help='Show this help message and exit.')
    parser.add_argument('-c', '--colors', type=int, choices=(1, 2, 3), default=3,
                        help="Full spectrum with broadband pulses or limited"
                        " to one or two colors (default: %(default)d).")
    parser.add_argument('-j', '--jmax', type=int,
                        help="Maximum initial angular momentum quantum number.")
    parser.add_argument('-k', '--kmax', type=int,
                        help="Maximum projection on principal molecular axis.")
    parser.add_argument('--no-abstract', action='store_true',
                        help='Print actual J values.')
    parser.add_argument('-f', "--filter", action='append',
                        help="Filter pathways by filtering excitation tree. "
                        "Can be provided multiple times to chain multiple filters."
                        "List of filter: https://allisonlab.gitlab.io/mdcs/rotsim2d/api/pathways.html#tree-filtering-functions")
    parser.add_argument('-a', '--angles', nargs=4, default=['0.0']*4,
                        help="Three beam angles and the detection angle. "
                        "Each angle can be a Python mathematical expression"
                        " using standard arithmetic operators and math"
                        " functions from Python math module.")
    parser.add_argument('-t', '--time', type=float, default=1.0,
                        help="Waiting time in ps (default: %(default)f).")
    parser.add_argument('-D', '--dpi', type=float,
                        help="Force DPI.")
    parser.add_argument('--symmetric-log', action='store_true',
                        help="Use symmetric logarithmic scaling for color"
                        " normalization.")
    args = parser.parse_args()

    aeval = Interpreter(use_numpy=False, minimal=True)
    angles = [aeval(angle) for angle in args.angles]
    if args.dpi:
        mpl.rcParams['figure.dpi'] = args.dpi

# * Vibrational mode
    print('Initializing vibrational mode')
    if args.molecule == 'CH3Cl':
        vib_mode = CH3ClAlchemyMode()
    else:
        vib_mode = COAlchemyMode()
    T = 296.0

# * Pathways
    print('Calculating peak list')
    jmax = args.jmax
    if jmax is None:
        if args.molecule == 'CH3Cl':
            jmax = 37
        else:
            jmax = 20

# ** Filters
    meths = []
    if args.colors == 2:
        meths.append(pw.remove_threecolor)
    elif args.colors == 1:
        meths.append(pw.only_dfwm)
    if args.filter:
        meths.extend([getattr(pw, filter) for filter in args.filter])
# ** Calculate peaks
    if args.kmax:
        kiter_func = "range((j if j<={kmax:d} else {kmax:d})+1)".format(kmax=args.kmax)
    else:
        kiter_func = "range(j+1)"
    pws = pw.gen_pathways(
        range(jmax), meths=meths,
        rotor='symmetric' if args.molecule == 'CH3Cl' else 'linear',
        kiter_func=kiter_func)
    dressed_pws = dl.DressedPathway.from_kb_list(pws, vib_mode, T)
    peaks = dl.Peak2DList.from_dp_list(
        dressed_pws, tw=args.time*1e-12, angles=angles)
    vminmax = np.max(np.abs(np.array(peaks.intensities)))*1.1*1e6

# * Visualize
    if args.symmetric_log:
        norm = colors.SymLogNorm(linthresh=vminmax/100.0,
                                 vmin=-vminmax, vmax=vminmax)
    else:
        norm = colors.Normalize(vmin=-vminmax, vmax=vminmax)

    fig = plt.figure(constrained_layout=True)
    gs = fig.add_gridspec(nrows=1, ncols=2, width_ratios=[20, 1])
    ax = fig.add_subplot(gs[0])
    sc = ax.scatter(peaks.probes, peaks.pumps, s=10.0,
                    c=-np.array(peaks.intensities)*1e6,
                    cmap=get_cmap('RdBu').reversed(), norm=norm, picker=True)
    ax.set(xlabel=r'$\Omega_3$ (cm$^{-1}$)',
           ylabel=r'$\Omega_1$ (cm$^{-1}$)')

    axcbar = fig.add_subplot(gs[-1])
    cbar = Colorbar(mappable=sc, ax=axcbar, orientation='vertical', extend='neither')
    amp_str = r"$S^{(3)}\cos \Omega_2 t_2$"
    cbar.set_label(amp_str + r" ($10^{-6}$ m$^{2}$ Hz/(V s/m)$^2$)")

    ax.set_title(str(args.filter), fontsize=10)
    fig.canvas.set_window_title(str(args.filter))

    abstract = not args.no_abstract
    def scatter_onpick(event):
        """Show information about the peak pathway."""
        if event.artist != sc:
            return
        dl.pprint_dllist(peaks[event.ind[0]].dp_list, abstract=abstract, angles=angles)


    fig.canvas.mpl_connect('pick_event', scatter_onpick)
    plt.show()



import sys

import matplotlib as mpl
import numpy as np
import PyQt5
import rotsim2d.dressedleaf as dl
import rotsim2d.pathways as pw
import rotsim2d.symbolic.functions as sym
from PyQt5 import Qt, QtCore, QtGui, QtWidgets

from .AngleWidget import Ui_AngleWidget
from .PolarizationsUI import Ui_MainWindow


class Model:
    def __init__(self, direction):
        kbs = pw.gen_pathways([5], rotor='symmetric',
                              meths=[getattr(pw, 'only_'+direction)],
                              kiter_func="[1]",
                              pump_overlap=False)
        pws = dl.Pathway.from_kb_list(kbs)
        rf_pws = sym.RFactorPathways.from_pwlist(pws, True, True)
        self.rfactors = [rfpw.rfactor for rfpw in rf_pws]
        self.titles = ['peaks: '+','.join(rfpw.peak_labels)+'\n'+
                       'trans: '+','.join(rfpw.trans_labels_deg)
                       for rfpw in rf_pws]
        self.angles = np.linspace(-np.pi/2, np.pi/2, 100)
        self.axes_labels = (r'$\Phi_2$', r'$\Phi_3$', r'$\Phi_4$')

    def plot_data(self, index, val):
        args = [0, self.angles[:, None], self.angles[None, :]]
        args.insert(index, val*np.pi/180.0)
        data = [rfactor.numeric_rel(*args) for rfactor in self.rfactors]

        return data

    def plot_axes_labels(self, index):
        labels = list(self.axes_labels)
        del labels[index]
        return labels


class AngleWidget(QtWidgets.QWidget, Ui_AngleWidget):
    def __init__(self, label, enabled=False, parent=None):
        super(AngleWidget, self).__init__(parent)
        self.setupUi(self)
        self.label.setTextFormat(QtCore.Qt.TextFormat.RichText)
        self.label.setText(label)
        self.slider.valueChanged.connect(self.spin.setValue)
        self.spin.valueChanged.connect(self.slider.setValue)
        self.radio.toggled.connect(self.slider.setEnabled)
        self.radio.toggled.connect(self.spin.setEnabled)
        self.radio.setChecked(enabled)


class Polarizations(QtWidgets.QMainWindow):
    def __init__(self, parent=None):
        QtWidgets.QMainWindow.__init__(self, parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        # take a reasonable amount of screen size
        screen = QtWidgets.QDesktopWidget().availableGeometry()
        self.resize(screen.width()*0.7, screen.height()*0.7)

        # add angle widgets
        self._radio_group = QtWidgets.QButtonGroup()
        self._radio_group.buttonToggled.connect(
            self._radio_toggled)
        self._angle_widgets = []
        self.ui.anglesLayout.setAlignment(QtCore.Qt.AlignmentFlag.AlignTop)
        self._add_angle_widget(AngleWidget("Φ<sub>2</sub>", True))
        self._add_angle_widget(AngleWidget("Φ<sub>3</sub>"))
        self._add_angle_widget(AngleWidget("Φ<sub>4</sub>"))
        self.ui.anglesLayout.setSpacing(10)

        # add direction radios
        self.ui.anglesLayout.insertSpacing(0, 10)
        self._dir_group = QtWidgets.QButtonGroup()
        self._dir_group.buttonToggled.connect(
            self._dir_toggled)
        self._add_dir_radio(QtWidgets.QRadioButton("SIII: k1+k2-k3"))
        btn = QtWidgets.QRadioButton("SII: k1-k2+k3")
        btn.setChecked(True)
        self._add_dir_radio(btn)
        self._add_dir_radio(QtWidgets.QRadioButton("SI: -k1+k2+k3"))

        # add model
        self.ui.statusbar.showMessage("Initializing model")
        self.model = Model('SII')
        self.ui.mplwdg.set_titles(self.model.titles)
        # self.update_plots()
        self.ui.statusbar.clearMessage()

    def _angle_index(self):
        index = 1
        for i in range(len(self._angle_widgets)):
            if self._angle_widgets[i].radio.isChecked():
                index += i

        return index

    def update_plots(self, val=None):
        index = self._angle_index()
        if val is None:
            val = self._angle_widgets[index-1].spin.value()
        data = self.model.plot_data(index, val)
        self.ui.mplwdg.figure_update(data)

    def update_axes_labels(self):
        index = self._angle_index()
        labels = self.model.plot_axes_labels(index-1)
        self.ui.mplwdg.set_axes_labels(labels)

    def _radio_toggled(self, button, checked):
        self.update_plots()
        self.update_axes_labels()

    def _add_angle_widget(self, wdg):
        wdg.spin.valueChanged.connect(self.update_plots)
        self._angle_widgets.append(wdg)
        self.ui.anglesLayout.addWidget(wdg)
        self._radio_group.addButton(wdg.radio)

    def _dir_toggled(self, button, checked):
        if checked:
            direction = button.text().split(':')[0]
            self.model = Model(direction)
            self.ui.mplwdg.set_titles(self.model.titles)
            self.update_plots()

    def _add_dir_radio(self, wdg):
        self.ui.anglesLayout.insertWidget(0, wdg)
        self._dir_group.addButton(wdg)

def run():
    app = QtWidgets.QApplication(sys.argv)
    mpl.rcParams['figure.dpi'] = app.desktop().physicalDpiX()
    polarizations = Polarizations()
    polarizations.show()
    sys.exit(app.exec_())

# Local Variables:
# compile-comand: "make -k"
# End:
